#---
# Excerpted from "Advanced Rails Recipes",
# published by The Pragmatic Bookshelf.
# Copyrights apply to this code. It may not be used to create training material, 
# courses, books, articles, and the like. Contact us if you are in doubt.
# We make no guarantees that this code is fit for any purpose. 
# Visit http://www.pragmaticprogrammer.com/titles/fr_arr for more book information.
#---
class CreditCard < ActiveRecord::Base
  include Encryption, ActiveMerchant::Billing::CreditCardMethods

  has_many :purchases
  cattr_accessor :password
  attr_accessor :number
    
  def verification_value?() false end
  
  def to_xml(options = {})
    super options.merge(:except => [:data, :salt])
  end
  
  def encrypt_number
    self.data = encrypt(number, password, salt)
  end

  def decrypt_number
    self.number = decrypt(data, password, salt)
  end
  
  private
  
  before_create :store_last_digits, :generate_salt, :encrypt_number

  def store_last_digits
    self.last_digits = self.class.last_digits(number)
  end
  
  def generate_salt
    self.salt = [rand(2**64 - 1)].pack("Q")
  end
  
  def validate
    errors.add(:year, "is invalid") unless valid_expiry_year?(year)
    errors.add(:month, "is invalid") unless valid_month?(month)
    errors.add(:number, "is invalid") unless self.class.valid_number?(number)
    if password.blank?
      errors.add_to_base("Unable to encrypt or decrypt data without password")
    end
  end
  
  validates_presence_of :first_name, :last_name, :brand
end
