#---
# Excerpted from "Advanced Rails Recipes",
# published by The Pragmatic Bookshelf.
# Copyrights apply to this code. It may not be used to create training material, 
# courses, books, articles, and the like. Contact us if you are in doubt.
# We make no guarantees that this code is fit for any purpose. 
# Visit http://www.pragmaticprogrammer.com/titles/fr_arr for more book information.
#---
require File.dirname(__FILE__) + '/../test_helper'

class CreditCardTest < ActiveSupport::TestCase

  def setup
    
    CreditCard.password = 'My really secret password'
    
    @credit_card = CreditCard.new(
                    :first_name => 'Cody',
                    :last_name => 'Fauser',
                    :number => '4242424242424242',
                    :month => '8',
                    :year => '2008',
                    :brand => 'visa'
                  )
  end

  def test_last_digits
    assert @credit_card.save
    assert_equal '4242', @credit_card.last_digits
  end
  
  def test_credit_card_validation
    assert_valid @credit_card
  end
  
  def test_invalid_credit_card_number
    @credit_card.number = 'invalid'
    assert !@credit_card.valid?
    assert @credit_card.errors.on(:number)
  end
  
  def test_to_xml_does_not_include_number
    card_hash = Hash.from_xml(@credit_card.to_xml)
    assert_nil card_hash["credit_card"]["number"]
  end
  
  def test_encrypt_and_decrypt_card_number
    assert @credit_card.save
    @credit_card.number = nil
    assert_nil @credit_card.number
    
    @credit_card.decrypt_number
    assert_equal '4242424242424242', @credit_card.number
  end
  
  def test_decrypting_visa
    credit_card = credit_cards(:visa)
    credit_card.decrypt_number
    assert_equal '4242424242424242', credit_card.number
  end
  
  def test_decrypting_master_card
    credit_card = credit_cards(:mastercard)
    credit_card.decrypt_number
    assert_equal '5105105105105100', credit_card.number
  end
  
  def test_invalid_password
    CreditCard.password = 'Whoops'
    
    assert_raises(OpenSSL::CipherError) do
      credit_cards(:visa).decrypt_number
    end
  end
end
