#---
# Excerpted from "Advanced Rails Recipes",
# published by The Pragmatic Bookshelf.
# Copyrights apply to this code. It may not be used to create training material, 
# courses, books, articles, and the like. Contact us if you are in doubt.
# We make no guarantees that this code is fit for any purpose. 
# Visit http://www.pragmaticprogrammer.com/titles/fr_arr for more book information.
#---
require File.dirname(__FILE__) + '/../test_helper'

class PurchaseTest < ActiveSupport::TestCase
  
  def setup
    CreditCard.password = 'My really secret password'
  end
  
  def test_valid_purchase
    credit_card = credit_cards(:visa)
    purchase = credit_card.purchases.build(
                :amount => '1.00',
                :order => '1000',
                :description => 'Monthly fee'
               )
    assert_valid purchase
  end
  
  def test_successful_purchase
    credit_card = credit_cards(:visa)
    
    assert_difference 'Purchase.count' do
      purchase = credit_card.purchases.build(
                  :amount => '1.00',
                  :order => '1000',
                  :description => 'Monthly fee'
                 )
                 
      assert purchase.save
    
      assert !purchase.reference.blank?
      assert !purchase.message.blank?
      assert purchase.test?
    end
  end
  
  def test_unsuccessful_purchase
    credit_card = credit_cards(:invalid)
    
    assert_no_difference 'Purchase.count' do
      purchase = credit_card.purchases.build(
                  :amount => '1.00',
                  :order => '1000',
                  :description => 'Monthly fee'
                 )
      
      assert !purchase.save
      assert !purchase.reference.blank?
      assert !purchase.message.blank?
      assert purchase.test?
    end
  end
  
  def test_purchase_with_invalid_password
    CreditCard.password = 'I forgot'
    
    credit_card = credit_cards(:visa)
    
    assert_raises(OpenSSL::CipherError) do
      credit_card.purchases.create(
        :amount => '1.00',
        :order => '1000',
        :description => 'Monthly fee'
      )
    end
  end
end