#---
# Excerpted from "Advanced Rails Recipes",
# published by The Pragmatic Bookshelf.
# Copyrights apply to this code. It may not be used to create training material, 
# courses, books, articles, and the like. Contact us if you are in doubt.
# We make no guarantees that this code is fit for any purpose. 
# Visit http://www.pragmaticprogrammer.com/titles/fr_arr for more book information.
#---
set :application, "robot"
set :repository,  "your-repo"

set :deploy_to, "/Library/Sites/#{application}"
set :deploy_via, :copy

role :app, "localhost", :cron => true
role :web, "localhost"
role :db,  "localhost", :primary => true


namespace :cron do

  task :start, :roles => :app, :only => {:cron => true} do
    cron_tab = "#{shared_path}/cron.tab"
    run "mkdir -p #{shared_path}/log/cron"
    
    require 'erb'
    template = File.read("config/cron.erb")
    file = ERB.new(template).result(binding)
    put file, cron_tab, :mode => 0644
    
    # merge with the current crontab
    # fails with an empty crontab, which is acceptable
    run "crontab -l >> #{cron_tab}" rescue nil
    
    # install the new crontab
    run "crontab #{cron_tab}"
  end
end



namespace :cron do

  task :stop, :roles => :app, :only => {:cron => true} do
    cron_tmp = "#{shared_path}/cron.old"
    cron_tab = "#{shared_path}/cron.tab"
    
    begin
      # dump the current cron entries
      run "crontab -l > #{cron_tmp}"
      
      # remove any lines that contain the application name
      run "awk '{if ($0 !~ /#{application}/) print $0}' " +
        "#{cron_tmp} > #{cron_tab}"
        
      # replace the cron entries
      run "crontab #{cron_tab}"
    rescue
      # fails with an empty crontab, which is acceptable
    end
    
    # clean up
    run "rm -rf #{cron_tmp}"
  end
  
  
end



before "deploy:stop",  "cron:stop"
after  "deploy:start", "cron:start"

