#pragma D option quiet
self int depth;
  
/*
  BEGIN is a probe that is fired every time dtrace starts up.
  We're using it here to print a convenient reminder of how to 
  stop tracing.
*/
dtrace:::BEGIN
{
  printf("Tracing... Hit Ctrl-C to stop.\n");
}

/*
  We're only interested in watching Cart#add_product, so we start
  looking for that method.  When it fires we'll set a variable
  indicating that we want to start watching things.
*/
/* START:one */
ruby*:::function-entry
/copyinstr(arg0) == "Cart" && copyinstr(arg1) == "add_product"/
{
  self->profile_it = 1;
}
/* END:one */

/*
  When Cart#add_product returns we are no longer interested in
  tracing anything so we set our variable switch to off.
*/
/* START:two */
ruby*:::function-return
/copyinstr(arg0) == "Cart" && copyinstr(arg1) == "add_product"/
{
  self->profile_it = 0;
}
/* END:two */

/*
  Once we've entered Cart#add_product we want to watch every
  method that runs.  This probe specification watches all methods
  using function-entry but notice the conditional block.  We
  only take action if our profile_it switch is on, meaning we're
  inside Cart#add_product.

  When this probe fires we record a timestamp for later use in
  determining how long the method took to run.
*/
/* START:three */
ruby*:::function-entry
/self->profile_it == 1/
{
  self->depth++;
  self->function[self->depth] = timestamp;
}
/* END:three */

/*
  When a method returns while we're tracing Cart#add_product
  (notice the conditional) we do the actual bookkeeping.
*/
/* START:four */
ruby*:::function-return
/self->profile_it == 1/
{
  /* Measure how long the method took. */
  this->elapsed = timestamp - self->function[self->depth];
  self->function[self->depth] = 0;

  /* Save the method name.  Example: "CartItem::initialize". */
  this->name = strjoin(strjoin(copyinstr(arg0), "::"), copyinstr(arg1));

  /* Count the number of times this method was called */
  @method_count[this->name] = count();

  /* Count the total number of methods called */
  @method_count["total"] = count();

  /* Sum the total amount of time this method has used */
  @method_time[this->name] = sum(this->elapsed);
  
  self->depth--;
}
/* END:four */

/*
  Like BEGIN, END is a probe that fires when the DTrace program is
  exiting.  Here we are just printing out the data we gathered into
  an easy to read report.
*/
/* START:five */
dtrace:::END
{
  printf("\nMethod Count,\n");
  printf("   %-64s %8s\n", "NAME", "TOTAL");
  printa("   %-64s %@8d\n", @method_count);
  
  normalize(@method_time, 1000);
  printf("\nElapsed times(us),\n");
  printf("   %-64s %8s\n", "NAME", "TOTAL");
  printa("   %-64s %@8d\n", @method_time);
}
/* END:five */
