#---
# Excerpted from "Advanced Rails Recipes",
# published by The Pragmatic Bookshelf.
# Copyrights apply to this code. It may not be used to create training material, 
# courses, books, articles, and the like. Contact us if you are in doubt.
# We make no guarantees that this code is fit for any purpose. 
# Visit http://www.pragmaticprogrammer.com/titles/fr_arr for more book information.
#---

class InvitationsTest < ActionController::IntegrationTest
  fixtures :users, :invitations

  def setup
    ActionMailer::Base.deliveries.clear
  end

  def test_should_allow_creating_invitations
    new_session_as 'alex' do
      get '/admin/invitations'
      assert_select %(form[action=/admin/invitations][method=post]) do
        assert_select %(input[type=text][name='invitation[email]'])
      end

      assert_difference 'Invitation.count', 1 do
        assert_difference "ActionMailer::Base.deliveries.size", 1 do
          post '/admin/invitations', 
               :invitation => {:email => 'joe@example.com'}
          assert_redirected_to '/admin/invitations'
          assert_match(/sent/i, session['flash'][:notice])
          assert_not_nil(invitation = 
            Invitation.find_by_email('joe@example.com'))
          assert_match(%r{https://spreedly.dev/signup/#{invitation.code}}, 
                       ActionMailer::Base.deliveries.last.body)
        end
      end
    end
  end
  
  
  def test_should_fill_code
    get '/signup/bob'
    assert_select %(input[name=code][value=bob])
  end

  def test_should_require_code
    assert_no_difference "Invitation.count" do
      post '/signup', :user => {:email => 'bob@example.com', :login => 'bobby', :password => 'testtest', :password_confirmation => 'testtest'}, :production_site => { :url_suffix => 'bob' }, :test_site => { :url_suffix => 'bob-test' }
      assert_response :success

      post '/signup', :code => 'nonexistentcode', :user => {:email => 'bob@example.com', :login => 'bobby', :password => 'testtest', :password_confirmation => 'testtest'}, :production_site => { :url_suffix => 'bob' }, :test_site => { :url_suffix => 'bob-test' }
      assert_response :success
    end

    assert_difference "Invitation.count", -1 do
      post '/signup', :code => 'bob', :user => {:email => 'bob@example.com', :login => 'bobby', :password => 'testtest', :password_confirmation => 'testtest'}, :production_site => { :url_suffix => 'bob' }, :test_site => { :url_suffix => 'bob-test' }
      assert_response :redirect
    end
  end

  
  
  def test_should_list_outstanding_invitations
    new_session_as 'alex' do
      get '/admin/invitations'
      assert_select 'li', :text => /bob@example\.com/ do
        resend_url = "/admin/invitations/#{invitations(:bob).id}/resend"
        assert_select %(a[href=#{resend_url}])
      end
    end
  end

  def test_should_allow_resending
    new_session_as 'alex' do
      assert_difference "ActionMailer::Base.deliveries.size", 1 do
        post "/admin/invitations/#{invitations(:bob).id}/resend"
        assert_redirected_to '/admin/invitations'
      end
    end
  end
  

  def test_should_allow_deleting
    new_session_as 'alex' do
      assert_difference "Invitation.count", -1 do
        delete "/admin/invitations/#{invitations(:bob).id}"
        assert_redirected_to '/admin/invitations'
      end
    end
  end

  def test_should_control_access
    get '/admin/invitations'
    assert_redirected_to '/sessions/new'

    new_session_as 'duff' do
      get '/admin/invitations'
      assert_redirected_to '/production'
    end

    new_session_as 'alex' do
      get '/admin/invitations'
      assert_response :success
    end
  end


end
