#---
# Excerpted from "Advanced Rails Recipes",
# published by The Pragmatic Bookshelf.
# Copyrights apply to this code. It may not be used to create training material, 
# courses, books, articles, and the like. Contact us if you are in doubt.
# We make no guarantees that this code is fit for any purpose. 
# Visit http://www.pragmaticprogrammer.com/titles/fr_arr for more book information.
#---
require File.dirname(__FILE__) + '/../spec_helper'

describe PeopleController do
  
  before do
    @person = mock_model(Person)
    @params = {'name' => 'Mike Clark',
               'url' => 'http://clarkware.com'}
  end
  
  it "should redirect to the new person on create" do
    Person.should_receive(:create!).with(@params).and_return(@person)
    
    post 'create', {:person => @params}
    
    response.should redirect_to(person_url(@person))
  end
  
  it "should load a person on read" do
    Person.stub!(:find).and_return(@person)
    
    get 'show', :id => @person.id
    
    response.should be_success
    assigns[:person].should eql(@person)
  end
  
  it "should modify a person on update" do
    @person = Person.create(:name => "Schmike Schlark", 
                            :url => 'http://schlarkware.com')
    Person.should_receive(:find).and_return(@person)
    @person.should_receive(:update_attributes!).with(@params)
    
    put 'update', :id => @person.id, :person => @params
    
    response.should be_success
    assigns[:person].should eql(@person)
  end
  
  it "should destroy a person on delete" do
    @person = Person.create(:name => "Schmike Schlark", 
                            :url => 'http://schlarkware.com')
    Person.should_receive(:find).and_return(@person)
    @person.should_receive(:destroy)
    
    delete 'destroy', :id => @person.id
    
    response.should be_redirect
    response.should redirect_to(people_path)
  end
  
  it "should list people on index" do
    @people = [Person.create(:name => 'Jack Shaftoe', 
                             :url => 'http://quicksilver.name'),
               Person.create(:name => 'Bob Shaftoe',
                             :url => 'http://respectableshaftoe.name')]
    Person.should_receive(:find).and_return(@people)
    
    get 'index'
    
    response.should be_success
    assigns[:people].should eql(@people)
  end
  
  it "should load a new person on new" do
    Person.should_receive(:new).and_return(@person)
    
    get 'new'
    
    response.should be_success
    assigns[:person].should eql(@person)
  end
  
  it "should load a person on edit" do
    Person.stub!(:find).and_return(@person)
    
    get 'edit', :id => @person.id
    
    response.should be_success
    assigns[:person].should eql(@person)
  end
end
