#---
# Excerpted from "Advanced Rails Recipes",
# published by The Pragmatic Bookshelf.
# Copyrights apply to this code. It may not be used to create training material, 
# courses, books, articles, and the like. Contact us if you are in doubt.
# We make no guarantees that this code is fit for any purpose. 
# Visit http://www.pragmaticprogrammer.com/titles/fr_arr for more book information.
#---
require File.dirname(__FILE__) + '/../spec_helper'

describe ZodiacSignsController do
  describe "handling GET /zodiac_signs" do

    before(:each) do
      @zodiac_sign = mock_model(ZodiacSign)
      ZodiacSign.stub!(:find).and_return([@zodiac_sign])
    end
  
    def do_get
      get :index
    end
  
    it "should be successful" do
      do_get
      response.should be_success
    end

    it "should render index template" do
      do_get
      response.should render_template('index')
    end
  
    it "should find all zodiac_signs" do
      ZodiacSign.should_receive(:find).with(:all).and_return([@zodiac_sign])
      do_get
    end
  
    it "should assign the found zodiac_signs for the view" do
      do_get
      assigns[:zodiac_signs].should == [@zodiac_sign]
    end
  end

  describe "handling GET /zodiac_signs.xml" do

    before(:each) do
      @zodiac_sign = mock_model(ZodiacSign, :to_xml => "XML")
      ZodiacSign.stub!(:find).and_return(@zodiac_sign)
    end
  
    def do_get
      @request.env["HTTP_ACCEPT"] = "application/xml"
      get :index
    end
  
    it "should be successful" do
      do_get
      response.should be_success
    end

    it "should find all zodiac_signs" do
      ZodiacSign.should_receive(:find).with(:all).and_return([@zodiac_sign])
      do_get
    end
  
    it "should render the found zodiac_signs as xml" do
      @zodiac_sign.should_receive(:to_xml).and_return("XML")
      do_get
      response.body.should == "XML"
    end
  end

  describe "handling GET /zodiac_signs/1" do

    before(:each) do
      @zodiac_sign = mock_model(ZodiacSign)
      ZodiacSign.stub!(:find).and_return(@zodiac_sign)
    end
  
    def do_get
      get :show, :id => "1"
    end

    it "should be successful" do
      do_get
      response.should be_success
    end
  
    it "should render show template" do
      do_get
      response.should render_template('show')
    end
  
    it "should find the zodiac_sign requested" do
      ZodiacSign.should_receive(:find).with("1").and_return(@zodiac_sign)
      do_get
    end
  
    it "should assign the found zodiac_sign for the view" do
      do_get
      assigns[:zodiac_sign].should equal(@zodiac_sign)
    end
  end

  describe "handling GET /zodiac_signs/1.xml" do

    before(:each) do
      @zodiac_sign = mock_model(ZodiacSign, :to_xml => "XML")
      ZodiacSign.stub!(:find).and_return(@zodiac_sign)
    end
  
    def do_get
      @request.env["HTTP_ACCEPT"] = "application/xml"
      get :show, :id => "1"
    end

    it "should be successful" do
      do_get
      response.should be_success
    end
  
    it "should find the zodiac_sign requested" do
      ZodiacSign.should_receive(:find).with("1").and_return(@zodiac_sign)
      do_get
    end
  
    it "should render the found zodiac_sign as xml" do
      @zodiac_sign.should_receive(:to_xml).and_return("XML")
      do_get
      response.body.should == "XML"
    end
  end

  describe "handling GET /zodiac_signs/new" do

    before(:each) do
      @zodiac_sign = mock_model(ZodiacSign)
      ZodiacSign.stub!(:new).and_return(@zodiac_sign)
    end
  
    def do_get
      get :new
    end

    it "should be successful" do
      do_get
      response.should be_success
    end
  
    it "should render new template" do
      do_get
      response.should render_template('new')
    end
  
    it "should create an new zodiac_sign" do
      ZodiacSign.should_receive(:new).and_return(@zodiac_sign)
      do_get
    end
  
    it "should not save the new zodiac_sign" do
      @zodiac_sign.should_not_receive(:save)
      do_get
    end
  
    it "should assign the new zodiac_sign for the view" do
      do_get
      assigns[:zodiac_sign].should equal(@zodiac_sign)
    end
  end

  describe "handling GET /zodiac_signs/1/edit" do

    before(:each) do
      @zodiac_sign = mock_model(ZodiacSign)
      ZodiacSign.stub!(:find).and_return(@zodiac_sign)
    end
  
    def do_get
      get :edit, :id => "1"
    end

    it "should be successful" do
      do_get
      response.should be_success
    end
  
    it "should render edit template" do
      do_get
      response.should render_template('edit')
    end
  
    it "should find the zodiac_sign requested" do
      ZodiacSign.should_receive(:find).and_return(@zodiac_sign)
      do_get
    end
  
    it "should assign the found ZodiacSign for the view" do
      do_get
      assigns[:zodiac_sign].should equal(@zodiac_sign)
    end
  end

  describe "handling POST /zodiac_signs" do

    before(:each) do
      @zodiac_sign = mock_model(ZodiacSign, :to_param => "1")
      ZodiacSign.stub!(:new).and_return(@zodiac_sign)
    end
    
    describe "with successful save" do
  
      def do_post
        @zodiac_sign.should_receive(:save).and_return(true)
        post :create, :zodiac_sign => {}
      end
  
      it "should create a new zodiac_sign" do
        ZodiacSign.should_receive(:new).with({}).and_return(@zodiac_sign)
        do_post
      end

      it "should redirect to the new zodiac_sign" do
        do_post
        response.should redirect_to(zodiac_sign_url("1"))
      end
      
    end
    
    describe "with failed save" do

      def do_post
        @zodiac_sign.should_receive(:save).and_return(false)
        post :create, :zodiac_sign => {}
      end
  
      it "should re-render 'new'" do
        do_post
        response.should render_template('new')
      end
      
    end
  end

  describe "handling PUT /zodiac_signs/1" do

    before(:each) do
      @zodiac_sign = mock_model(ZodiacSign, :to_param => "1")
      ZodiacSign.stub!(:find).and_return(@zodiac_sign)
    end
    
    describe "with successful update" do

      def do_put
        @zodiac_sign.should_receive(:update_attributes).and_return(true)
        put :update, :id => "1"
      end

      it "should find the zodiac_sign requested" do
        ZodiacSign.should_receive(:find).with("1").and_return(@zodiac_sign)
        do_put
      end

      it "should update the found zodiac_sign" do
        do_put
        assigns(:zodiac_sign).should equal(@zodiac_sign)
      end

      it "should assign the found zodiac_sign for the view" do
        do_put
        assigns(:zodiac_sign).should equal(@zodiac_sign)
      end

      it "should redirect to the zodiac_sign" do
        do_put
        response.should redirect_to(zodiac_sign_url("1"))
      end

    end
    
    describe "with failed update" do

      def do_put
        @zodiac_sign.should_receive(:update_attributes).and_return(false)
        put :update, :id => "1"
      end

      it "should re-render 'edit'" do
        do_put
        response.should render_template('edit')
      end

    end
  end

  describe "handling DELETE /zodiac_signs/1" do

    before(:each) do
      @zodiac_sign = mock_model(ZodiacSign, :destroy => true)
      ZodiacSign.stub!(:find).and_return(@zodiac_sign)
    end
  
    def do_delete
      delete :destroy, :id => "1"
    end

    it "should find the zodiac_sign requested" do
      ZodiacSign.should_receive(:find).with("1").and_return(@zodiac_sign)
      do_delete
    end
  
    it "should call destroy on the found zodiac_sign" do
      @zodiac_sign.should_receive(:destroy)
      do_delete
    end
  
    it "should redirect to the zodiac_signs list" do
      do_delete
      response.should redirect_to(zodiac_signs_url)
    end
  end
end