#---
# Excerpted from "Advanced Rails Recipes",
# published by The Pragmatic Bookshelf.
# Copyrights apply to this code. It may not be used to create training material, 
# courses, books, articles, and the like. Contact us if you are in doubt.
# We make no guarantees that this code is fit for any purpose. 
# Visit http://www.pragmaticprogrammer.com/titles/fr_arr for more book information.
#---
require 'digest/sha1'

class User < ActiveRecord::Base

  attr_accessor :password_confirmation
  
  attr_accessible :login, :email, :password
  
  validates_presence_of     :login, :email
  validates_uniqueness_of   :login, :email
  validates_format_of       :email, :with => /(\S+)@(\S+)/, :allow_nil => true
  validates_confirmation_of :password
  validates_length_of       :password, :minimum => 4, :allow_nil => true

  has_many :registrations, :dependent => :destroy
  has_many :events, :through => :registrations, :order => 'events.starts_at'

  has_and_belongs_to_many :tags
  
  def register_for(event)
    return false if self.events.find_by_id(event.id)
    registration = event.registrations.build(:user => self)
    registration.save
  end
            
  def validate
    errors.add_to_base("Missing password") if hashed_password.blank?
  end
                  
  def self.authenticate(login, password)
    user = self.find_by_login(login)
    (user && user.authenticated?(password)) ? user : nil    
  end

  def authenticated?(password)
    self.hashed_password == User.encrypted_password(password, self.salt)
  end
  
  # 'password' is a virtual attribute
  def password
    @password
  end
  
  def password=(pwd)
    @password = pwd
    create_new_salt
    self.hashed_password = User.encrypted_password(self.password, self.salt)
  end
  
  def to_xml(options = {})
    default_except = [:salt, :hashed_password]
    options[:except] = (options[:except] || []) + default_except
    super(options)
  end
  
private

  def create_new_salt
    self.salt = self.object_id.to_s + rand.to_s
  end
  
  def self.encrypted_password(password, salt)
    Digest::SHA1.hexdigest(password + "wibble" + salt)
  end

end
