#---
# Excerpted from "Advanced Rails Recipes",
# published by The Pragmatic Bookshelf.
# Copyrights apply to this code. It may not be used to create training material, 
# courses, books, articles, and the like. Contact us if you are in doubt.
# We make no guarantees that this code is fit for any purpose. 
# Visit http://www.pragmaticprogrammer.com/titles/fr_arr for more book information.
#---
require File.dirname(__FILE__) + '/../test_helper'

class UserTest < ActiveSupport::TestCase

  def setup
    @valid_attributes = { 
        :name => "Wilma Flintstone",
        :email => 'wilma@flintstones.com', 
        :password => 'secret', 
        :password_confirmation => 'secret'
      }
    
    @user = User.new(@valid_attributes)
  end

  def test_advanced_fixtures    
    assert_equal ["Caveman", "Programmer"], users(:fred).tags.map(&:name)
    assert_equal ["Caveman", "Juggler"], users(:barney).tags.map(&:name)
    
    assert_equal 10.00, users(:fred).registrations.first.price
  end


  def test_create
    assert_difference('User.count') do
      user = User.create(@valid_attributes)
    end
  end
  
  def test_not_valid_without_required_fields
    user = User.new
    assert !user.valid?
    [:name, :email].each do |field|
      assert user.errors.invalid?(field), field
    end
  end
    
  def test_required_fields
    [:name, :email].each do |field|
      u = User.new(@valid_attributes.merge(field => nil))
      assert !u.valid?, "#{field} should be invalid"
      assert u.errors.invalid?(field)
    end
  end

  def test_require_valid_email
    @user.email = nil
    
    @user.email = "abc"
    assert !@user.valid?

    @user.email = "abc@de"
    assert @user.valid?, @user.errors.full_messages
  end

  def test_require_password_confirmation
    @user.password_confirmation = ''
  
    assert !@user.valid?
    assert @user.errors.invalid?(:password)
  end

  def test_require_unique_email
    @user.email = users(:fred).email

    assert !@user.valid?
    assert @user.errors.invalid?(:email)
  end

  def test_require_reasonable_password_length
    @user.password, @user.password_confirmation = "123"

    assert !@user.valid?
    assert_equal "is too short (minimum is 4 characters)", @user.errors.on(:password)
  end  

  def test_authenticate
    assert_equal users(:fred), User.authenticate(users(:fred).email, 'test')
  end

end
