#---
# Excerpted from "Advanced Rails Recipes",
# published by The Pragmatic Bookshelf.
# Copyrights apply to this code. It may not be used to create training material, 
# courses, books, articles, and the like. Contact us if you are in doubt.
# We make no guarantees that this code is fit for any purpose. 
# Visit http://www.pragmaticprogrammer.com/titles/fr_arr for more book information.
#---

require 'resolv'

class Account < ActiveRecord::Base
  
  EMAIL_PATTERN = /(\S+)@(\S+)/



  validates_format_of :email, :with => EMAIL_PATTERN
  
  def validate
    unless errors.on(:email)
      unless valid_domain?(email)
        errors.add(:email, 'domain name appears to be incorrect') 
      end
    end
  end



  SERVER_TIMEOUT = 3 # seconds

  def valid_domain?(email)
    domain = email.match(EMAIL_PATTERN)[2]
    
    dns = Resolv::DNS.new

    Timeout::timeout(SERVER_TIMEOUT) do
      # Check the MX records
      mx_records = 
        dns.getresources(domain, Resolv::DNS::Resource::IN::MX)
    
      mx_records.sort_by {|mx| mx.preference}.each do |mx|
        a_records = dns.getresources(mx.exchange.to_s, 
                                     Resolv::DNS::Resource::IN::A)
        return true if a_records.any?
      end
      
      # Try a straight A record
      a_records = dns.getresources(domain, Resolv::DNS::Resource::IN::A)
      a_records.any?
    end
  rescue Timeout::Error, Errno::ECONNREFUSED
    false
  end
end

