#---
# Excerpted from "Advanced Rails Recipes",
# published by The Pragmatic Bookshelf.
# Copyrights apply to this code. It may not be used to create training material, 
# courses, books, articles, and the like. Contact us if you are in doubt.
# We make no guarantees that this code is fit for any purpose. 
# Visit http://www.pragmaticprogrammer.com/titles/fr_arr for more book information.
#---

class Event < ActiveRecord::Base
  
  validates_presence_of   :name, :description, :image_location, 
                          :starts_at, :location, :capacity
  validates_uniqueness_of :name

  validates_numericality_of :price, :greater_than_or_equal_to => 0.0
  
  has_many :registrations
  has_many :attendees, :through => :registrations, 
                       :source => :user
  
  
  validates_format_of :image_location, 
                      :with => %r{\.(gif|jpg|png)$}i, 
                      :message => "must be a GIF, JPG, or PNG image"
                        
  def self.find_upcoming_events(options={}) 
    upcoming_scope = {
      :conditions => ['starts_at >= ?', Time.now], 
      :order      => 'starts_at'
    }
    with_scope(:find => upcoming_scope) do
      find(:all, options)
    end     
  end

  
  def free?
    self.price == 0
  end
  
  
  def spaces_remaining
    self.capacity - self.registrations.size
  end
  
  def sold_out?
    spaces_remaining <= 0
  end
  

end

