#---
# Excerpted from "Advanced Rails Recipes",
# published by The Pragmatic Bookshelf.
# Copyrights apply to this code. It may not be used to create training material, 
# courses, books, articles, and the like. Contact us if you are in doubt.
# We make no guarantees that this code is fit for any purpose. 
# Visit http://www.pragmaticprogrammer.com/titles/fr_arr for more book information.
#---
require File.dirname(__FILE__) + '/../test_helper'



class EventTest < ActiveSupport::TestCase



  
  should_require_attributes :name, :description, :image_location, 
                            :starts_at, :location, :capacity
  
  should_require_unique_attributes :name
  

  
  should_only_allow_numeric_values_for :price
  
  should_not_allow_values_for :price, -1.0, 
    :message => /must be greater than or equal to 0/ 
  

  
  should_have_many :attendees, :through => :registrations
  
    
  
  
  context "An event" do
    
    setup do
      @event = events(:railsconf)
    end
  
  
    should "be free when the price is $0" do 
      @event.price = 0 
      assert_equal true, @event.free? 
    end
  
    should "not be free when the price isn't $0" do
      @event.price = 1.0 
      assert_equal false, @event.free? 
    end  
    
    

    context "yesterday" do
      setup do
        @event.starts_at = 1.day.ago
        assert_save @event
      end
      
      should "not be upcoming" do
        assert_does_not_contain Event.find_upcoming_events, @event
      end
    end

    context "tomorrow" do
      setup do
        @event.starts_at = 1.day.from_now
        assert_save @event
      end
      
      should "be upcoming" do
        assert_contains Event.find_upcoming_events, @event
      end
    end

    
    
    context "with excess capacity" do
      
      setup do
        @event.capacity = 2
        @event.registrations = [registrations(:fred_for_rails)]
      end
    
      should "have spaces remaining" do
        assert_equal 1, @event.spaces_remaining
      end
      
      should "not be sold out" do
        assert_equal false, @event.sold_out?
      end
    end
    

    context "with full capacity" do
      setup do
        @event.capacity = 1
        @event.registrations = [registrations(:fred_for_rails)]
      end
    
      should "not have spaces remaining" do
        assert_equal 0, @event.spaces_remaining
      end
      
      should "be sold out" do
        assert_equal true, @event.sold_out?
      end
    end
   
    should_eventually "should pass" do
      flunk "what?"
    end
  
  
  end
  
  
  


end



