#---
# Excerpted from "Advanced Rails Recipes",
# published by The Pragmatic Bookshelf.
# Copyrights apply to this code. It may not be used to create training material, 
# courses, books, articles, and the like. Contact us if you are in doubt.
# We make no guarantees that this code is fit for any purpose. 
# Visit http://www.pragmaticprogrammer.com/titles/fr_arr for more book information.
#---
class ItemsController < ApplicationController
  before_filter :populate_list
  
  # GET /items
  # GET /items.xml
  def index
    @items = @list.items.find(:all)

    respond_to do |format|
      format.html # index.html.erb
      format.xml  { render :xml => @items }
    end
  end

  # GET /items/1
  # GET /items/1.xml
  def show
    @item = @list.items.find(params[:id])

    respond_to do |format|
      format.html # show.html.erb
      format.iphone { render :layout => false } # show.iphone.erb
      format.xml  { render :xml => @item }
    end
  end

  # GET /items/new
  # GET /items/new.xml
  def new
    @item = @list.items.new

    respond_to do |format|
      format.html # new.html.erb
      format.iphone  { render :layout => false } # new.iphone.erb
      format.xml  { render :xml => @item }
    end
  end

  # GET /items/1/edit
  def edit
    @item = @list.items.find(params[:id])
  end

  # POST /items
  # POST /items.xml
  def create
    @item = @list.items.new(params[:item])

    respond_to do |format|
      if @item.save
        flash[:notice] = 'Item was successfully created.'
        format.html { redirect_to list_path(@list) }
        format.iphone  { redirect_to list_path(@list) }
        format.xml  { render :xml => @item, :status => :created, :location => @item }
      else
        format.html { render :action => "new" }
        format.iphone { render :action => "new", :layout => false }
        format.xml  { render :xml => @item.errors, :status => :unprocessable_entity }
      end
    end
  end

  # PUT /items/1
  # PUT /items/1.xml
  def update
    @item = @list.items.find(params[:id])

    respond_to do |format|
      if @item.update_attributes(params[:item])
        flash[:notice] = 'Item was successfully updated.'
        format.html { redirect_to list_path(@list) }
        # format.iphone { redirect_to list_path(@list) }
        format.iphone { render :nothing => true }
        format.xml  { head :ok }
      else
        format.html { render :action => "edit" }
        format.iphone { render :action => "edit", :layout => false }
        format.xml  { render :xml => @item.errors, :status => :unprocessable_entity }
      end
    end
  end

  # DELETE /items/1
  # DELETE /items/1.xml
  def destroy
    @item = @list.items.find(params[:id])
    @item.destroy

    respond_to do |format|
      format.html { redirect_to list_path(@list) }
      format.xml  { head :ok }
    end
  end
  
private

  def populate_list
    @list = List.find(params[:list_id])
  end
end